/*
 * DAVID
 * 
 * This file modified by David Foster to improve clarity.
 */

/*
 * Otsu Thresholding algorithm
 *
 * Copyright (c) 2003 by Christopher Mei (christopher.mei@sophia.inria.fr)
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.util.*;
import ij.*;
import ij.process.*;

/** 
 *  This class implements a class of pixels with basic statistical functions.
 **/

public class GrayLevelClass {
    private static float[] probabilityHistogram;
    public static boolean probabilityHistogramDone;
    public static int N;

    private int index;
    private float omega;
    private float mu;

    public GrayLevelClass(ByteProcessor img, boolean first) {
        if(!probabilityHistogramDone) {
            int[] histogram = img.getHistogram();
            probabilityHistogram = new float[256];
            
            for(int i=0; i<256 ; i++) {
                probabilityHistogram[i] = ((float) histogram[i])/((float) N);
            }
            probabilityHistogramDone = true;
        }

        if(first) {
            index = 1;
            
            /*
             * omega = mass(probabilityHistogram[i], 0, 0);
             * mu = center_of_mass(probabilityHistogram[i], 0, 0);
             */
            omega = probabilityHistogram[index-1];
            mu = (omega == 0)
                ? 0
                : (probabilityHistogram[index-1]/omega);
        }
        else {
            index = 2;
            
            magic_calcMuAndOmega(index);	// DAVID
        }
    }

    public void removeFromBeginning() {
        index++;
        
        magic_calcMuAndOmega(index);		// DAVID
    }

    public void addToEnd() {
        index++;
        
        magic_calcMuAndOmega(1);			// DAVID
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        
        ret.append("Index : "+index+"\n");
        ret.append("Mu : "+mu+"\n");
        ret.append("Omega : "+omega+"\n");
        
        return ret.toString();
    }

    public float getMu() {
        return mu;
    }

    public float getOmega() {
        return omega;
    }

    public int getThreshold() {
        return index;
    }
    
    /*
     * DAVID: saw this code snippet repeated many times
     *
     * Behavior:
     * > omega = summation(probabilityHistogram[x], x, startIndex-1, 254);
     * > mu = (omega != 0)
     *       ? summation(x*probabilityHistogram[x], x, startIndex-1, 254)/omega
     *       : 0;
     */
    private void magic_calcMuAndOmega(int startIndex) {
        /*
         * (After consulting my calculus book...)
         * 
         * Let mu = mass(probabilityHistogram[i], startIndex-1, 254);
         * Let omega = first_moment(probabilityHistogram[i], startIndex-1, 254);
         */
        mu = 0;
		omega = 0;
		for(int i=startIndex; i<256 ; i++) {
			omega +=  probabilityHistogram[i-1];
			mu +=  i*probabilityHistogram[i-1];
		}
		
		/*
		 * (center_of_mass = mass / first_moment)
		 * 
		 * Let omega = center_of_mass(probabilityHistogram[i], startIndex-1, 254);
		 */
		mu = (omega == 0)
			? 0
			: (mu / omega);
    }
}

